package com.gitlab.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.symmetric.AES;
import com.alibaba.fastjson.JSONObject;
import com.gitlab.config.OAuth2Properties;
import com.gitlab.constant.OAuth2Constants;
import com.gitlab.exception.BizException;
import com.gitlab.model.*;
import com.gitlab.util.JWTUtils;
import com.gitlab.util.OAuth2SecurityUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Base64;
import java.util.Date;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @Description OAuth2Controller 类（或接口）
 * @Author huangshun
 * @Date 2024/4/22
 */
@Slf4j
@Tag(name = "OAuth2认证服务")
@Controller
public class OAuth2Controller {

    @Autowired
    private OAuth2SecurityUtils securityUtils;

    @Autowired
    private OAuth2Properties properties;

    @Value(value = "${oauth2.redirect_login}")
    private String redirectLogin;

    @RequestMapping("/oauth2/authorize")
    @Operation(summary = "认证授权")
    public String authorize(HttpServletRequest request, AuthorizeRequestDTO requestDTO) {
        UserinfoDTO userinfoDTO = this.getUserinfo(request);
        if (BeanUtil.isEmpty(userinfoDTO)) {
            return String.format("redirect:%s", redirectLogin);
        }
        String data = JSONObject.toJSONString(userinfoDTO);
        try {
            data = securityUtils.encryptByPublicKey(data);
            // 计算code超时时间
            DateTime dateTime = DateUtil.offset(new Date(), DateField.SECOND, this.properties.getCodeExpires());
            String code = JWTUtils.create(this.properties.getSinglePrivateKey(), userinfoDTO.getUsername(), data, dateTime.getTime());
            String redirect_uri = requestDTO.getRedirect_uri();
            String redirect = "";
            if (redirect_uri.contains("?")) {
                redirect = String.format("%s&code=%s", redirect_uri, code);
            }
            else {
                redirect = String.format("%s?code=%s", redirect_uri, code);
            }
            if(StringUtils.hasLength(requestDTO.getState())){
                redirect = String.format("%s&state=%s", redirect, requestDTO.getState());
            }
            return String.format("redirect:%s", redirect);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BizException("认证授权异常!");
        }
        // auap开发逻辑
        /**
         * 认证授权接口地址：/auth/oauth2/authorize
         * 请求参数：
         */
    }

    @ResponseBody
    @PostMapping("/oauth2/token")
    @Operation(summary = "获取AccessToken")
    public AccessTokenDTO token(TokenRequestDTO requestDTO) {
        if (!StringUtils.hasLength(requestDTO.getClient_id())) {
            throw new BizException("client_id is null");
        }
        if (!StringUtils.hasLength(requestDTO.getClient_secret())) {
            throw new BizException("client_secret is null");
        }
        if (!StringUtils.hasLength(requestDTO.getCode())) {
            throw new BizException("code is null");
        }
        AccessTokenDTO tokenDTO = new AccessTokenDTO();
        try {
            if (!JWTUtils.verify(requestDTO.getCode(), this.properties.getSinglePrivateKey())) {
                throw new BizException("认证失败，凭证非法，请重新认证!");
            }
            AtomicReference<Long> expire = new AtomicReference<>();
            AtomicReference<String> sub = new AtomicReference<>();
            String data = (String) JWTUtils.of(requestDTO.getCode(), jwt -> {
                expire.set((Long) jwt.getPayload("exp"));
                sub.set((String) jwt.getPayload("sub"));
                return jwt.getPayload("data");
            });

            if (expire.get().compareTo(System.currentTimeMillis()) < 0) {
                throw new BizException("认证失败，凭证过期，请重新认证!");
            }
            // 计算token超时时间
            DateTime accessTokenDateTime = DateUtil.offset(new Date(), DateField.SECOND, this.properties.getTokenExpires());
            DateTime refreshTokenDateTime = DateUtil.offset(new Date(), DateField.SECOND, this.properties.getTokenExpires() + this.properties.getTokenExpires());
            String accessToken = JWTUtils.create(this.properties.getSinglePrivateKey(), sub.get(), data, accessTokenDateTime.getTime());
            String refreshToken = JWTUtils.create(this.properties.getSinglePrivateKey(), sub.get(), data, refreshTokenDateTime.getTime());

            tokenDTO = new AccessTokenDTO()
                    .setAccess_token(accessToken)
                    .setRefresh_token(refreshToken)
                    .setId_token(accessToken)
                    .setToken_type(OAuth2Constants.HEADER_AUTHORIZATION_BEARER.replace(" ", ""))
                    .setExpires_in(DateUtil.offset(new Date(), DateField.HOUR, 8).getTime());

        } catch (Exception e) {
           e.printStackTrace();
           throw new BizException(e.getMessage(),e.getCause());
        }
        return tokenDTO;
    }

    @ResponseBody
    @RequestMapping("/userinfo")
    @Operation(summary = "获取userinfo用户信息")
    public UserinfoDTO profile(HttpServletRequest request) {
        String access_token = request.getHeader(OAuth2Constants.HEADER_AUTHORIZATION_NAME);
        if (!StringUtils.hasLength(access_token)) {
            log.error("认证失败，参数为空!");
            return null;
        }
        try {
            access_token = access_token.replace(OAuth2Constants.HEADER_AUTHORIZATION_BEARER, "");
            if (!JWTUtils.verify(access_token, this.properties.getSinglePrivateKey())) {
                throw new BizException("认证失败，凭证非法，请重新认证!");
            }
            AtomicReference<Long> expire = new AtomicReference<>();
            UserinfoDTO userinfoDTO = JWTUtils.of(access_token, jwt -> {
                expire.set((Long) jwt.getPayload("exp"));
                String data = (String) jwt.getPayload("data");
                try {
                    String decodeValue = this.securityUtils.decryptByPrivateKey(data);
                    if (!StringUtils.hasLength(decodeValue) || !(decodeValue.startsWith("{") && decodeValue.endsWith("}"))) {
                        return null;
                    }
                    return JSONObject.parseObject(decodeValue, UserinfoDTO.class);
                } catch (Exception e) {
                    e.printStackTrace();
                    return null;
                }

            });

            if (Objects.isNull(userinfoDTO)) {
                throw new BizException("认证失败，参数非法!");
            }

            if (expire.get().compareTo(System.currentTimeMillis()) < 0) {
                throw new BizException("认证失败，凭证过期，请重新认证!");
            }
//            userinfoDTO.setEmail(userinfoDTO.getUsername());
            return userinfoDTO;
        } catch (Exception e) {
            e.printStackTrace();
            log.error("认证失败，系统出现异常，请联系系统管理员");
            return null;
        }
    }

    private UserinfoDTO getUserinfo(HttpServletRequest request) {
        UserinfoDTO userinfoDTO = new UserinfoDTO();
        userinfoDTO.setUserId("2");
        userinfoDTO.setRealname("张三");
        // 该字段jfrog使用（但是传了email后该字段的值会失效，填充的内容将是email）
        userinfoDTO.setUsername("zhangsan");
        userinfoDTO.setEnName("zhangsan");
        userinfoDTO.setEmail("365152541@qq.com");
        return userinfoDTO;
    }

    public static void main(String[] args) {
        System.out.println(Base64.getEncoder().encodeToString("ssoAdmin:Admin@123".getBytes()));
    }

}
