package com.gitlab.exception;

import cn.hutool.core.util.StrUtil;
import com.gitlab.enums.BasicErrorCode;
import com.gitlab.model.ResultEntity;
import com.gitlab.util.ServletUtils;
import com.gitlab.util.ValidatorUtils;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.validation.ConstraintViolation;
import jakarta.validation.ConstraintViolationException;
import jakarta.validation.ValidationException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.multipart.MultipartException;

import java.util.Objects;
import java.util.Set;

/**
* @Description GlobalExceptionHandler 类（或接口）
* @Author huangshun
* @Date 2024/4/22
*/
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    /**
     * 400 - Bad Request
     */
    @ExceptionHandler(value = {MethodArgumentNotValidException.class,
            BindException.class,
            HttpMessageNotReadableException.class,
            ConstraintViolationException.class,
            ValidationException.class
    })
    @ResponseStatus(HttpStatus.OK)
    public ResultEntity<?> handleMethodArgumentNotValidException(Exception e) {
        log.error("参数验证失败：", e);
        String message;
        if (e instanceof MethodArgumentNotValidException || e instanceof BindException) {
            BindingResult result;
            if (e instanceof MethodArgumentNotValidException) {
                result = ((MethodArgumentNotValidException) e).getBindingResult();
            } else {
                result = ((BindException) e).getBindingResult();
            }
            FieldError error = result.getFieldError();
            assert error != null;
            message = error.getDefaultMessage();
        } else {
            if (e instanceof HttpMessageNotReadableException) {
                message = "参数解析失败";
            } else if (e instanceof ConstraintViolationException) {
                ConstraintViolationException ex = (ConstraintViolationException) e;
                Set<ConstraintViolation<?>> violations = ex.getConstraintViolations();
                ConstraintViolation<?> violation = violations.iterator().next();
                message = violation.getMessage();
            } else if (e instanceof ValidationException) {
                message = "参数验证失败: " + e.getMessage();
            } else {
                message = e.getMessage();
            }
        }
        return ResultEntity.fail(BasicErrorCode.PARAM_ERROR.getCode(), message);
    }


    /**
     * 业务异常处理方法
     */
    @ExceptionHandler(value = BizException.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultEntity<?> bizException(BizException e) {
        log.error(StrUtil.format("业务异常:{}", getApiAndTenantInfo()), e);
        return ResultEntity.fail(e.getCode(), e.getMessage());
    }

    /**
     * 系统异常处理方法
     */
    @ExceptionHandler(value = SysException.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultEntity<?> sysException(SysException e) {
        log.error(StrUtil.format("系统异常:{}", getApiAndTenantInfo()), e);
        return ResultEntity.fail(e.getCode(), e.getMessage());
    }

    /**
     * 405 - Method Not Allowed
     */
    @ResponseStatus(HttpStatus.METHOD_NOT_ALLOWED)
    @ExceptionHandler(value = HttpRequestMethodNotSupportedException.class)
    public ResultEntity<?> handleHttpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException e) {
        return ResultEntity.fail(BasicErrorCode.SERVER_ERROR.getCode(), "当前接口不支持【" + e.getMethod() + "】方法！");
    }


    /**
     * 通用未知异常
     */
    @ExceptionHandler(Throwable.class)
    @ResponseStatus(HttpStatus.OK)
    public ResultEntity<?> error(Exception e) {
        log.error(StrUtil.format("系统错误:{}", getApiAndTenantInfo()), e);
        if (Objects.nonNull(e.getCause()) && e.getCause().getCause() instanceof BizException) {
            BizException bizException = (BizException) e.getCause().getCause();
            return bizException(bizException);
        }
        if (Objects.nonNull(e.getCause()) && e.getCause().getCause() instanceof SysException) {
            SysException sysException = (SysException) e.getCause().getCause();
            return sysException(sysException);
        }
        if(Objects.nonNull(e) && Objects.nonNull(e.getMessage())){
            if(ValidatorUtils.isContainChinese(e.getMessage())){
                return ResultEntity.fail(BasicErrorCode.UNKNOWN_ERROR, e.getMessage());
            }
        }
        return ResultEntity.fail(BasicErrorCode.SERVER_ERROR.getCode(), "系统错误，请联系管理员！");
    }

    /**
     * 文件过大 RequestTooBigException.class
     */
    @ExceptionHandler({MultipartException.class})
    @ResponseStatus(HttpStatus.OK)
    public ResultEntity<?> requestTooBigException(MultipartException e) {
        log.error(StrUtil.format("文件上传错误:{}", getApiAndTenantInfo()), e);
        return ResultEntity.fail(BasicErrorCode.SERVER_ERROR.getCode(), "文件过大，无法上传");
    }

    /**
     * 运行时业务异常处理方法
     */
    @ExceptionHandler(value = {RuntimeException.class})
    @ResponseStatus(HttpStatus.OK)
    public ResultEntity<?> runtimeException(RuntimeException e) {
        log.error(StrUtil.format("业务异常:{}", getApiAndTenantInfo()), e);
        if(e instanceof BaseException || (Objects.nonNull(e.getCause()) && e.getCause() instanceof BaseException) || (Objects.nonNull(e.getCause()) && Objects.nonNull(e.getCause().getCause()) && e.getCause().getCause() instanceof BaseException)){
            BaseException baseException = null;
            if(e instanceof BaseException){
                baseException = (BaseException) e;
            }else if(Objects.nonNull(e.getCause()) && e.getCause() instanceof BaseException) {
                baseException = (BaseException) e.getCause();
            }else{
                baseException = (BaseException) e.getCause().getCause();
            }

            ErrorCode errorCode = Objects.nonNull(baseException.getErrorCode()) ? baseException.getErrorCode() : BasicErrorCode.UNKNOWN_ERROR;
            String errorMsg = StringUtils.hasLength(baseException.getMessage()) ? baseException.getMessage() : errorCode.getMessage();
            return ResultEntity.fail(errorCode, errorMsg);
        }
        if(Objects.nonNull(e) && Objects.nonNull(e.getMessage())){
            if(ValidatorUtils.isContainChinese(e.getMessage())){
                return ResultEntity.fail(BasicErrorCode.UNKNOWN_ERROR, e.getMessage());
            }
        }
        return ResultEntity.fail(BasicErrorCode.UNKNOWN_ERROR, BasicErrorCode.UNKNOWN_ERROR.getMessage());
    }

    /**
     * 通用业务异常处理方法
     */
    @ExceptionHandler(value = {Exception.class})
    @ResponseStatus(HttpStatus.OK)
    public ResultEntity<?> exception(Exception e) {
        log.error(StrUtil.format("业务异常:{}", getApiAndTenantInfo()), e);
        if(Objects.nonNull(e.getCause()) && Objects.nonNull(e.getCause().getCause()) && e.getCause().getCause() instanceof BaseException){
            BaseException baseException = (BaseException) e.getCause().getCause();
            ErrorCode errorCode = Objects.nonNull(baseException.getErrorCode()) ? baseException.getErrorCode() : BasicErrorCode.UNKNOWN_ERROR;
            String errorMsg = StringUtils.hasLength(baseException.getMessage()) ? baseException.getMessage() : BasicErrorCode.UNKNOWN_ERROR.getMessage();
            return ResultEntity.fail(errorCode, errorMsg);
        }
        if(Objects.nonNull(e) && Objects.nonNull(e.getMessage())){
            if(ValidatorUtils.isContainChinese(e.getMessage())){
                return ResultEntity.fail(BasicErrorCode.UNKNOWN_ERROR, e.getMessage());
            }
        }
        return ResultEntity.fail(BasicErrorCode.UNKNOWN_ERROR, BasicErrorCode.UNKNOWN_ERROR.getMessage());
    }

    private String getApiAndTenantInfo() {
        String api = null;
        HttpServletRequest request = ServletUtils.getRequest();
        if (request != null) {
            api = request.getRequestURI();
        }
        return StrUtil.format(" 接口:{};", api);
    }


}
