package com.gitlab.model;

import com.gitlab.enums.ResultEnum;
import com.gitlab.exception.ErrorCode;
import com.gitlab.enums.BasicErrorCode;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import org.springframework.http.HttpStatus;

import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;

/**
 * @Description ResultEntity 类（或接口）
 * @Author huangshun
 * @Date 2023/6/17
 */
@Data
@Schema(description = "统一返回对象")
public class ResultEntity<T> {
    private static final long serialVersionUID = 1L;

    /**
     * 响应编号
     */
    @Schema(description = "响应编号")
    private String code;

    /**
     * 响应信息
     */
    @Schema(description = "响应信息")
    private String msg;

    /**
     * 响应数据
     */
    @Schema(description = "响应数据")
    private T data;

    /**
     * 时间戳
     */
    @Schema(description = "时间戳")
    private Long ts = 0L;
    /**
     * 响应状态
     */
    @Schema(description = "响应状态")
    private boolean success = false;

    public boolean getSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public static <T> ResultEntity<T> ok() {
        ResultEntity<T> r = restResult(null, ResultEnum.SUCCESS.getCode(), null);
        r.setSuccess(true);
        return r;
    }

    public static <T> ResultEntity<T> ok(T data) {
        ResultEntity<T> r = restResult(data, ResultEnum.SUCCESS.getCode(), null);
        r.setSuccess(true);
        return r;
    }

    public static <T> ResultEntity<T> ok(T data, String msg) {
        ResultEntity<T> r = restResult(data, ResultEnum.SUCCESS.getCode(), msg);
        r.setSuccess(true);
        return r;
    }

    public static <T> ResultEntity<T> fail() {
        return restResult(null, BasicErrorCode.SERVER_ERROR.getCode(), null);
    }

    public static <T> ResultEntity<T> fail(String msg) {
        return restResult(null, BasicErrorCode.SERVER_ERROR.getCode(), msg);
    }

    public static <T> ResultEntity<T> fail(T data) {
        return restResult(data, BasicErrorCode.SERVER_ERROR.getCode(), null);
    }

    public static <T> ResultEntity<T> fail(ErrorCode errorCode) {
        return restResult(null, errorCode.getCode(), errorCode.getMessage());
    }

    public static <T> ResultEntity<T> fail(ErrorCode errorCode, String msg) {
        return restResult(null, errorCode.getCode(), msg);
    }

    public static <T> ResultEntity<T> fail(ErrorCode errorCode, T data) {
        return restResult(data, errorCode.getCode(), errorCode.getMessage());
    }

    public static <T> ResultEntity<T> fail(HttpStatus httpStatus, String msg) {
        return restResult(null, httpStatus.value() + "", msg);
    }

    public static <T> ResultEntity<T> fail(String msg, T data) {
        return restResult(data, BasicErrorCode.SERVER_ERROR.getCode(), msg);
    }

    public static <T> ResultEntity<T> fail(String code, String msg) {
        return restResult(null, code, msg);
    }


    private static <T> ResultEntity<T> restResult(T data, String code, String msg) {
        ResultEntity<T> apiResult = new ResultEntity<>();
        apiResult.setCode(code);
        apiResult.setData(data);
        apiResult.setMsg(msg);
        apiResult.setTs(System.currentTimeMillis());
        return apiResult;
    }

    public static <T> T data(ResultEntity<T> resultEntity, T defaultIfNull) {
        return Optional.ofNullable(resultEntity)
                .map(ResultEntity::getData)
                .orElse(defaultIfNull);
    }

    public static <T> Optional<T> data(ResultEntity<T> resultEntity) {
        return Optional.ofNullable(resultEntity)
                .map(ResultEntity::getData);
    }

    public static <T> T dataThrow(ResultEntity<T> resultEntity, RuntimeException e) {
        return Optional.ofNullable(resultEntity)
                .map(ResultEntity::getData)
                .orElseThrow(() -> e);
    }

    public static <T> void successThrow(ResultEntity<T> resultEntity, Function<Optional<String>, RuntimeException> exceptionFunction) {
        boolean success = Optional.ofNullable(resultEntity).map(ResultEntity::getSuccess).orElse(false);
        if (!success) {
            throw exceptionFunction.apply(Optional.ofNullable(resultEntity).map(ResultEntity::getMsg));
        }
    }

    /**
     * 如果失败就消费
     *
     * @param resultEntity resultEntity
     * @param failConsumer 失败如何消费
     * @param <T>          T
     */
    public static <T> void failConsumer(ResultEntity<T> resultEntity, Consumer<Optional<String>> failConsumer) {
        boolean success = Optional.ofNullable(resultEntity).map(ResultEntity::getSuccess).orElse(false);
        if (!success) {
            if (failConsumer != null) {
                failConsumer.accept(Optional.ofNullable(resultEntity).map(ResultEntity::getMsg));
            }
        }
    }

    public static <T> boolean isSuccess(ResultEntity<T> resultEntity) {
        return Optional.ofNullable(resultEntity).map(ResultEntity::getSuccess).orElse(false);
    }
}
