package com.gitlab.util;

import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Enumeration;
import java.util.LinkedHashMap;
import java.util.Map;

/** 
* @Description ServletUtils 类（或接口）
* @Author huangshun
* @Date 2023/7/18 
*/
@Slf4j
public class ServletUtils {

    /**
     * 获取request
     */
    public static HttpServletRequest getRequest() {
        try {
            ServletRequestAttributes requestAttributes = getRequestAttributes();
            return requestAttributes == null ? null : requestAttributes.getRequest();
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 获取response
     */
    public static HttpServletResponse getResponse() {
        try {
            ServletRequestAttributes requestAttributes = getRequestAttributes();
            return requestAttributes == null ? null : requestAttributes.getResponse();
        } catch (Exception e) {
            return null;
        }
    }

    public static ServletRequestAttributes getRequestAttributes() {
        try {
            RequestAttributes attributes = RequestContextHolder.getRequestAttributes();
            return (ServletRequestAttributes) attributes;
        } catch (Exception e) {
            return null;
        }
    }

    public static Map<String, String> getHeaders(HttpServletRequest request) {
        if (request == null) {
            return null;
        }
        Map<String, String> map = new LinkedHashMap<>(16);
        Enumeration<String> enumeration = request.getHeaderNames();
        if (enumeration != null) {
            while (enumeration.hasMoreElements()) {
                String key = enumeration.nextElement();
                String value = request.getHeader(key);
                map.put(key, value);
            }
        }
        return map;
    }

    public static Map<String, String> getHeaders() {
        return getHeaders(getRequest());
    }

    public static String getHeader(String key) {
        HttpServletRequest request = getRequest();
        if (request == null) {
            return null;
        }
        return request.getHeader(key);
    }

    /**
     * 是否是Ajax异步请求
     *
     * @param request req
     */
    public static boolean isAjaxRequest(HttpServletRequest request) {
        String accept = request.getHeader("accept");
        if (accept != null && accept.contains("application/json")) {
            return true;
        }

        String xRequestedWith = request.getHeader("X-Requested-With");
        if (xRequestedWith != null && xRequestedWith.contains("XMLHttpRequest")) {
            return true;
        }

        String uri = request.getRequestURI();
        if (StringUtils.inStringIgnoreCase(uri, ".json", ".xml")) {
            return true;
        }

        String ajax = request.getParameter("__ajax");
        return StringUtils.inStringIgnoreCase(ajax, "json", "xml");
    }

    /**
     * 对下载的文件名称进行转码
     *
     * @param filename filename
     * @param request  request
     * @param response response
     * @return return
     * @throws UnsupportedEncodingException e
     */
    public static String encodeFilename(String filename, HttpServletRequest request, HttpServletResponse response) throws UnsupportedEncodingException {
        String userAgent = request.getHeader("User-Agent");
        response.setCharacterEncoding("UTF-8");
        String encodeFilename;
        if (/* IE 8 至 IE 10 */
                userAgent.toUpperCase().contains("MSIE") ||
                        /* IE 11 */
                        userAgent.contains("Trident/7.0")) {
            encodeFilename = URLEncoder.encode(filename, StandardCharsets.UTF_8.toString());
        } else if (userAgent.toUpperCase().contains("MOZILLA") ||
                userAgent.toUpperCase().contains("CHROME")) {
            encodeFilename = new String(filename.getBytes(), StandardCharsets.ISO_8859_1);
        } else {
            encodeFilename = URLEncoder.encode(filename, StandardCharsets.UTF_8.toString());
        }
        return encodeFilename;
    }

    public static void exportTemplate(String excelTemplate, String templateName, HttpServletRequest request, HttpServletResponse response) {
        response.setContentType("application/vnd.ms-excel");
        try {
            response.setHeader("Content-Disposition", "attachment;filename=" + ServletUtils.encodeFilename(templateName, request, response) + ".xlsx");
            ClassPathResource resource = new ClassPathResource(excelTemplate);
            ServletOutputStream out = response.getOutputStream();
            InputStream is = resource.getInputStream();
            byte[] bis = new byte[1024];
            int n;
            while ((n = is.read(bis)) != -1) {
                out.write(bis, 0, n);
            }
            out.flush();
            out.close();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取body
     *
     * @param request 请求
     * @return body
     */
    public static String getBodyString(HttpServletRequest request) {
        StringBuilder sb = new StringBuilder();
        try (InputStream inputStream = request.getInputStream();
             BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8))) {
            String line;
            while ((line = reader.readLine()) != null) {
                sb.append(line);
            }
        } catch (IOException e) {
            log.error("body读取失败", e);
        }
        return sb.toString();
    }

}
