package com.gitlab.util;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.StringJoiner;

/**
 * @author: ShiYunXiao
 * @date: 2023/05/24 09:48
 */
public class StringUtils extends org.apache.commons.lang3.StringUtils {

    /**
     * 下划线
     */
    public static final String UNDERLINE = "_";

    /**
     * 下划线转驼峰命名
     */
    public static String toUnderScoreCase(String str) {
        if (str == null) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        // 前置字符是否大写
        boolean preCharIsUpperCase;
        // 当前字符是否大写
        boolean curreCharIsUpperCase;
        // 下一字符是否大写
        boolean nexteCharIsUpperCase = true;
        for (int i = 0; i < str.length(); i++) {
            char c = str.charAt(i);
            if (i > 0) {
                preCharIsUpperCase = Character.isUpperCase(str.charAt(i - 1));
            } else {
                preCharIsUpperCase = false;
            }

            curreCharIsUpperCase = Character.isUpperCase(c);

            if (i < (str.length() - 1)) {
                nexteCharIsUpperCase = Character.isUpperCase(str.charAt(i + 1));
            }

            if (preCharIsUpperCase && curreCharIsUpperCase && !nexteCharIsUpperCase) {
                sb.append(UNDERLINE);
            } else if (i != 0 && !preCharIsUpperCase && curreCharIsUpperCase) {
                sb.append(UNDERLINE);
            }
            sb.append(Character.toLowerCase(c));
        }

        return sb.toString();
    }

    /**
     * 是否包含字符串
     *
     * @param str     验证字符串
     * @param strList 字符串组
     * @return 包含返回true
     */
    public static boolean inStringIgnoreCase(String str, String... strList) {
        if (str != null && strList != null) {
            for (String s : strList) {
                if (str.equalsIgnoreCase(trim(s))) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 将下划线大写方式命名的字符串转换为驼峰式。如果转换前的下划线大写方式命名的字符串为空，则返回空字符串。 例如：HELLO_WORLD->HelloWorld
     *
     * @param name 转换前的下划线大写方式命名的字符串
     * @return 转换后的驼峰式命名的字符串
     */
    public static String convertToCamelCase(String name) {
        StringBuilder result = new StringBuilder();
        // 快速检查
        if (name == null || name.isEmpty()) {
            // 没必要转换
            return "";
        } else if (!name.contains(UNDERLINE)) {
            // 不含下划线，仅将首字母大写
            return name.substring(0, 1).toUpperCase() + name.substring(1);
        }
        // 用下划线将原始字符串分割
        String[] camels = name.split(UNDERLINE);
        for (String camel : camels) {
            // 跳过原始字符串中开头、结尾的下换线或双重下划线
            if (camel.isEmpty()) {
                continue;
            }
            // 首字母大写
            result.append(camel.substring(0, 1).toUpperCase());
            result.append(camel.substring(1).toLowerCase());
        }
        return result.toString();
    }

    /**
     * 驼峰式命名法 例如：user_name->userName
     */
    public static String toCamelCase(String s) {
        if (s == null) {
            return null;
        }
        s = s.toLowerCase();
        StringBuilder sb = new StringBuilder(s.length());
        boolean upperCase = false;
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);

            if (c == UNDERLINE.charAt(0)) {
                upperCase = true;
            } else if (upperCase) {
                sb.append(Character.toUpperCase(c));
                upperCase = false;
            } else {
                sb.append(c);
            }
        }
        return sb.toString();
    }

    /**
     * 从uri的query中获取所有参数
     *
     * @param queryStr uriQuery,如:userId=1&username=test
     * @return uriQuery中的参数, 如key:userId value:1
     */
    public static Map<String, String> getUriQueryMap(String queryStr) {
        Map<String, String> queryMap = new HashMap<>(16);
        if (!org.apache.commons.lang3.StringUtils.isEmpty(queryStr)) {
            String[] queryParam = queryStr.split("&");
            Arrays.stream(queryParam).forEach(s -> {
                String[] kv = s.split("=", 2);
                String value = kv.length == 2 ? kv[1] : "";
                queryMap.put(kv[0], value);
            });
        }
        return queryMap;
    }

    /**
     * 多个字符串通过分隔符分割组成字符串
     *
     * @param delimiter 分隔符
     * @param objects   字符串对象
     * @return 字符串
     */
    public static String join(CharSequence delimiter, Object... objects) {
        StringJoiner sj = new StringJoiner(delimiter);

        for (Object object : objects) {
            if (object != null) {
                sj.add(object.toString());
            }
        }

        return sj.toString();
    }

    /**
     * 左侧补0
     * totalLength为补充0后的总长度，必须>0，否则返回obj的原本内容
     * obj不能为空
     * example1: totalLength = [0,-1,-2] | obj = 123 | result = 123
     * example2: totalLength = [1,2,3] | obj = 123 | result = 123
     * example3: totalLength = 4 | obj = 123 | result = 0123
     *
     * @param totalLength 总长度
     * @param obj         需要补充0的对象
     * @return 左补0的字符串
     */
    public static String addZeroLeft(int totalLength, Object obj) {
        if (obj == null) {
            return null;
        }
        if (totalLength <= 0) {
            return obj.toString();
        }
        return String.format("%0" + totalLength + "d", obj);
    }

    /**
     * 按最大长度截取字符串
     *
     * @param str       字符串
     * @param maxLength 最大长度
     * @return 新字符串
     */
    public static String substringByMaxLength(String str, int maxLength) {
        if (str != null && str.length() > maxLength) {
            return str.substring(0, maxLength);
        }
        return str;
    }

    /**
     * Trim the trailing linebreak (if any) from this string.
     *
     * @param str 目标字符串
     * @return 替换后的字符串
     */
    public static String trimLineBreak(String str) {
        if (!hasLength(str)) {
            return str;
        }
        StringBuilder buf = new StringBuilder(str);
        while (buf.length() > 0 && isLineBreakCharacter(buf.charAt(buf.length() - 1))) {
            buf.deleteCharAt(buf.length() - 1);
        }
        return buf.toString();
    }

    private static boolean isLineBreakCharacter(char ch) {
        return '\n' == ch || '\r' == ch;
    }

    public static boolean hasLength(String str) {
        return str != null && str.length() > 0;
    }
}
